@@ -5,14 +5,14 @@ IMDB::Film - OO Perl interface to the movies database IMDB.
 =head1 SYNOPSIS
 
 	use IMDB::Film;
-	
+
 	#
 	# Retrieve a movie information by its IMDB code
 	#
 	my $imdbObj = new IMDB::Film(crit => 227445);
 
 	or
-	
+
 	#
 	# Retrieve a movie information by its title
 	#
@@ -42,6 +42,7 @@ You can use that module to retrieve information about film:
 title, year, plot etc. 
 
 =cut
+
 package IMDB::Film;
 
 use strict;
@@ -53,7 +54,7 @@ use Carp;
 use Data::Dumper;
 
 use fields qw(	_title
-		_kind
+				_kind
 				_year
 				_episodes
 				_episodeof
@@ -64,6 +65,7 @@ use fields qw(	_title
 				_cover
 				_language
 				_country
+				_top_info
 				_rating
 				_genres
 				_tagline
@@ -78,6 +80,15 @@ use fields qw(	_title
 				_official_sites
 				_release_dates
 				_aspect_ratio
+				_mpaa_info
+				_company
+				_connections
+				_full_companies
+				_recommendation_movies
+				_plot_keywords
+				_big_cover_url
+				_big_cover_page
+				_storyline
 				full_plot_url
 		);
 	
@@ -88,10 +99,10 @@ use constant FORCED			=> 1;
 use constant USE_CACHE		=> 1;
 use constant DEBUG_MOD		=> 1;
 use constant EMPTY_OBJECT	=> 0;
-use constant MAIN_TAG		=> 'h5';
+use constant MAIN_TAG		=> 'h4';
 
 BEGIN {
-		$VERSION = '0.34';
+		$VERSION = '0.53';
 						
 		# Convert age gradation to the digits		
 		# TODO: Store this info into constant file
@@ -122,15 +133,18 @@ BEGIN {
 		matched			=> [],
         host			=> 'www.imdb.com',
         query			=> 'title/tt',
-        search 			=> 'find?s=tt;q=',	
+        search 			=> 'find?s=tt&exact=true&q=',	
 		status			=> 0,		
 		timeout			=> 10,
 		user_agent		=> 'Mozilla/5.0',
+		decode_html		=> 1,
 		full_plot_url	=> 'http://www.imdb.com/rg/title-tease/plotsummary/title/tt',		
 		_also_known_as	=> [],
 		_official_sites	=> [],
 		_release_dates	=> [],
 		_duration		=> [],
+		_top_info		=> [],
+		_cast			=> [],
 	);	
 	
 	sub _get_default_attrs { keys %_defaults }		
@@ -164,6 +178,7 @@ For more infomation about base methods refer to IMDB::BaseClass.
 Initialize object.
 
 =cut
+
 sub _init {
 	my CLASS_NAME $self = shift;
 	my %args = @_;
@@ -180,7 +195,8 @@ sub _init {
 		return;
 	} 
 
-	for my $prop (grep { /^_/ && !/^(_title|_code|_full_plot|_official_sites|_release_dates)$/ } sort keys %FIELDS) {
+	for my $prop (grep { /^_/ &&
+	!/^(_title|_code|_full_plot|_official_sites|_release_dates|_connections|_full_companies|_plot_keywords|_big_cover_url|_big_cover_page)$/ } sort keys %FIELDS) {
 		($prop) = $prop =~ /^_(.*)/;
 		$self->$prop(FORCED);
 	}
@@ -290,11 +306,12 @@ sub fields {
 Implemets functionality to search film by name.
 
 =cut
+
 sub _search_film {
 	my CLASS_NAME $self = shift;
 	my $args = shift || {};
 
-	return $self->SUPER::_search_results('\/title\/tt(\d+)', '/td', $args->{year});
+	return $self->SUPER::_search_results('^\/title\/tt(\d+)', '/td', $args->{year});
 }
 
 =back
@@ -332,6 +349,7 @@ matched films and continue to process first one:
 	my $title = $film->title();
 
 =cut
+
 sub title {	
 	my CLASS_NAME $self = shift;
 	my $forced 	= shift || 0;
@@ -342,26 +360,31 @@ sub title {
 	
 		$parser->get_tag('title');
 		my $title = $parser->get_text();
-		if($title =~ /imdb\s+title\s+search/i) {
+		if($title =~ /Find \- IMDb/i) {
 			$self->_show_message("Go to search page ...", 'DEBUG');
 			$title = $self->_search_film($args);				
 		} 
 		
 		if($title) {
-			$self->retrieve_code($parser, '/pro.imdb.com/title/tt(\d+)') 
-														unless $self->code;
+			$self->retrieve_code($parser, 'http://www.imdb.com/title/tt(\d+)') unless $self->code;
 			$title =~ s/\*/\\*/g;
+			$title = $self->_decode_special_symbols($title);
 			
-			($self->{_title}, $self->{_year}, $self->{_kind}) = $title =~ m!(.*?)\s+\(([\d\?]{4}).*?\)(?:\s+\((.*?)\))?!;
-			$self->{_kind} = '' unless $self->{_kind};
-			
-        		#"The Series" An Episode (2005)
-			#"The Series" (2005)
-        		if( $self->{_title} =~ /\"[^\"]+\"(\s+.+\s+)?/ ) {
-        			$self->{_kind} = $1 ? 'E' : 'S';
-        		}
-        		
+			$self->_show_message("title: $title", 'DEBUG');
+
+			# TODO: implement parsing of TV series like ALF (TV Series 1986–1990)
+			$title =~ s/^imdb\s+\-\s+//i;
+			($self->{_title}, $self->{_year}, $self->{_kind}) = $title =~ m!(.*?)\s+\((\d{4})(?:/[IVX]+)\)(?:\s\((\w*)\))?!;
+			unless($self->{_title}) {
+				($self->{_title}, $self->{_kind}, $self->{_year}) = $title =~ m!(.*?)\s+\((.*?)?\s?([0-9\-]*\s?)\)!;
+			}
+			$self->{_kind} = 'Movie' unless $self->{_kind}; # Default kind should be movie
 			
+       		# "The Series" An Episode (2005)
+			# "The Series" (2005)
+       		if( $self->{_title} =~ /\"[^\"]+\"(\s+.+\s+)?/ ) {
+       			$self->{_kind} = $1 ? 'E' : 'S';
+       		}		
 		}	
 	}	
 	
@@ -371,29 +394,199 @@ sub title {
 =item kind()
 
 Get kind of movie:
-	
+
 	my $kind = $film->kind();
-	
+
 	Possible values are: 'movie', 'tv series', 'tv mini series', 'video game', 'video movie', 'tv movie', 'episode'.
 
 =cut
+
 sub kind {
 	my CLASS_NAME $self = shift;
-	return $FILM_KIND{$self->{_kind}};
+	return exists $FILM_KIND{$self->{_kind}} ? $FILM_KIND{$self->{_kind}} : lc($self->{_kind});
 }
 
 =item year()
 
 Get film year:
-	
+
 	my $year = $film->year();
 
 =cut
+
 sub year {
 	my CLASS_NAME $self = shift;
 	return $self->{_year};
 }
 
+=item connections()
+
+Retrieve connections for the movie as an arrays of hashes with folloeing structure
+
+ 	{ 
+ 		follows 		=> [ { id => <id>, title => <name>, year => <year>, ...,  } ],
+  		followed_by  	=> [ { id => <id>, title => <name>, year => <year>, ...,  } ],
+  		references 		=> [ { id => <id>, title => <name>, year => <year>, ...,  } ],
+  		referenced_in 	=> [ { id => <id>, title => <name>, year => <year>, ...,  } ],
+  		featured_in 	=> [ { id => <id>, title => <name>, year => <year>, ...,  } ],
+  		spoofed_by 		=> [ { id => <id>, title => <name>, year => <year>, ...,  } ],
+	}
+
+  	my %connections = %{ $film->connections() };
+
+=cut
+
+sub connections {
+  	my CLASS_NAME $self = shift;
+
+  	unless($self->{_connections}) {
+    	my $page;
+    	$page = $self->_cacheObj->get($self->code . '_connections') if $self->_cache;
+
+    	unless($page) {
+      		my $url = "http://". $self->{host} . "/" . $self->{query} .  $self->code . "/trivia?tab=mc";
+      		$self->_show_message("URL for movie connections is $url ...", 'DEBUG');
+
+      		$page = $self->_get_page_from_internet($url);
+      		$self->_cacheObj->set($self->code.'_connections', $page, $self->_cache_exp) if $self->_cache;
+    	}
+
+    	my $parser = $self->_parser(FORCED, \$page);
+
+    	my $group = undef;
+    	my %result;
+    	my @lookFor = ('h4');
+   	 	while (my $tag = $parser->get_tag(@lookFor)) {
+      		if ($tag->[0] eq 'h4') {
+        		$group = HTML::Entities::encode_entities($parser->get_text);
+				$group = lc($group);
+				$group =~ s/\s+/_/g;
+				$group =~ s/(&nbsp;|\?|\:)//;
+				$group =~ s/&amp;/and/;
+        		$result{$group} = [];
+        		@lookFor = ('h4', 'a', 'hr', 'hr/');
+      		} elsif ($tag->[0] eq 'a') {
+        		my $id;
+				($id)= $tag->[1]->{href} =~ /(\d+)/ if $tag->[1]->{href};
+        		my $name = $parser->get_trimmed_text;
+
+        		# Handle series episodes (usually in 'referenced' sections)
+        		my($series,$t,$s,$e) = ($name =~ /^(.*?): *(.*?) *\(?#(\d+)\.(\d+)\)?$/);
+          		$name = $series if defined $series;
+        	
+				$tag = $parser->get_tag('/a');
+				my $next = $parser->get_trimmed_text();
+				my %film = ('id' => $id, 'title' => $name);
+				if(defined $t) {
+					$film{'series_title'} = $t;
+					$film{'season'} = $s;
+					$film{'episode'} = $e;
+				}
+
+				$film{'year'} = $1 if $next =~ /\((\d{4})\)/;				
+				next if ($next =~ /\(VG\)/);
+				push @{$result{$group}}, \%film;
+      		} else {
+        		# Stop when we hit the divider
+        		last;
+      		}
+    	}
+    	
+		$self->{_connections} = \%result;
+  	}
+
+  	return $self->{_connections};
+}
+
+
+=item full_companies()
+
+Retrieve companies for the movie as an array where each item has following stucture:
+
+	{ 
+		production 		=> [ { name => <company name>, url => <imdb url>, extra => <specific task> } ],
+  		distributors  	=> [ { name => <company name>, url => <imdb url>, extra => <specific task> } ],
+ 	 	special_effects => [ { name => <company name>, url => <imdb url>, extra => <specific task> } ],
+  		other 			=> [ { name => <company name>, url => <imdb url>, extra => <specific task> } ],
+	}
+
+  my %full_companies = %{ $film->full_companies() };
+
+=cut
+
+sub full_companies {
+  	my CLASS_NAME $self = shift;
+
+  	unless($self->{_full_companies}) {
+    	my $page;
+    	$page = $self->_cacheObj->get($self->code . '_full_companies') if $self->_cache;
+
+    	unless($page) {
+      		my $url = "http://". $self->{host} . "/" . $self->{query} .  $self->code . "/companycredits";
+      		$self->_show_message("URL for company credits is $url ...", 'DEBUG');
+
+      		$page = $self->_get_page_from_internet($url);
+      		$self->_cacheObj->set($self->code.'_full_companies', $page, $self->_cache_exp) if $self->_cache;
+    	}
+
+   	 	my $parser = $self->_parser(FORCED, \$page);
+    	my $group = undef;
+    	my %result;
+    	my @lookFor = ('h2');
+    	while (my $tag = $parser->get_tag(@lookFor)) {
+      		if ($tag->[0] eq 'h2') {
+        		$group = $parser->get_text;
+        		$group =~ s/ compan(y|ies)//i;
+        		$group =~ tr/A-Z/a-z/;
+        		$group =~ s/\s+/_/g;
+        		$result{$group} = [];
+        		@lookFor = ('h2', 'a', 'hr', 'hr/');
+      		} elsif($tag->[0] eq 'a') {
+        	
+			my($url) = $tag->[1]->{href};
+			my $name = $parser->get_trimmed_text;
+
+				$tag = $parser->get_tag('/a');
+				my $next = $parser->get_trimmed_text();
+				$next =~ s/^[\t \xA0]+//; # nbsp comes out as \xA0
+				my %company = ( 'url' => $url,
+								'name' => $name,
+								'extra' => $next );
+				push @{$result{$group}}, \%company;
+			} else {
+				# Stop when we hit the divider
+				last;
+			}
+    	}
+    
+		$self->{_full_companies} = \%result;
+  	}
+
+  	return $self->{_full_companies};
+}
+
+=item company()
+
+Returns a list of companies given for a specified movie:
+
+  my $company = $film->company();
+
+or  
+
+ my @companies = $film->company();
+
+=cut
+
+sub company {
+  	my CLASS_NAME $self = shift;
+	
+	unless($self->{_company}) {
+  		my @companies = split /\s?\,\s?/, $self->_get_simple_prop('Production Co');
+		$self->{_company} = \@companies;
+	}	
+  	
+	return wantarray ? $self->{_company} : $self->{_company}[0];
+}
 
 =item episodes()
 
@@ -403,17 +596,18 @@ Retrieve episodes info list each element of which is hash reference for tv serie
 	my @episodes = @{ $film->episodes() };
 
 =cut
+
 sub episodes {
 	my CLASS_NAME $self = shift;
 
-	return if $self->kind ne "tv series";
+	return [] if !$self->kind or $self->kind !~ /tv serie/i;
 
 	unless($self->{_episodes}) {
 		my $page;
 		$page = $self->_cacheObj->get($self->code . '_episodes') if $self->_cache;
 
 		unless($page) {
-			my $url = "http://". $self->{host} . "/" . $self->{query} .  $self->code . "/episodes";
+			my $url = "http://". $self->{host} . "/" . $self->{query} .  $self->code . "/epcast";
 			$self->_show_message("URL for episodes is $url ...", 'DEBUG');
 
 			$page = $self->_get_page_from_internet($url);
@@ -421,16 +615,15 @@ sub episodes {
 		}
 
 		my $parser = $self->_parser(FORCED, \$page);
-		while(my $tag = $parser->get_tag('a')) {
+		while(my $tag = $parser->get_tag('h4')) {
 			my $id;
-			next unless $tag->[1]->{name} and $tag->[1]->{name} =~ /year/;
-			$parser->get_tag('h4');
-			my($season, $episode) = $parser->get_text =~ /Season\s+(.*?),\s+Episode\s+([^:]+)/;
+            my($season, $episode);
+            next unless(($season, $episode) = $parser->get_text =~ /Season\s+(.*?),\s+Episode\s+([^:]+)/); 
 			my $imdb_tag = $parser->get_tag('a');
 			($id) = $imdb_tag->[1]->{href} =~ /(\d+)/ if $imdb_tag->[1]->{href};
 			my $title = $parser->get_trimmed_text;
 			$parser->get_tag('b');
-			my($date) = $parser->get_trimmed_text =~ /Original Air Date:\s+(.*)/;
+			my($date) = $parser->get_trimmed_text;
 			$parser->get_tag('br');
 			my $plot = $parser->get_trimmed_text;
 			
@@ -456,11 +649,12 @@ Retrieve parent tv series list each element of which is hash reference for episo
 	my @tvseries = @{ $film->episodeof() };
 
 =cut
+
 sub episodeof {
    my CLASS_NAME $self = shift;
    my $forced = shift || 0;
 
-   return if $self->kind ne "episode";
+   return if !$self->kind or $self->kind ne "episode";
 
    if($forced) {
 	   my($episodeof, $title, $year, $episode, $season, $id);
@@ -498,13 +692,14 @@ Retrieve url of film cover:
 	my $cover = $film->cover();
 
 =cut
+
 sub cover {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
 
 	if($forced) {
-		my ($parser) = $self->_parser(FORCED);
-		my ($cover);
+		my $parser = $self->_parser(FORCED);
+		my $cover;
 
 		my $title = quotemeta($self->title);
 		while(my $img_tag = $parser->get_tag('img')) {
@@ -512,7 +707,7 @@ sub cover {
 		
 			last if $img_tag->[1]{alt} =~ /^poster not submitted/i;			
 
-			if($img_tag->[1]{alt} =~ /^$title$/i) {
+			if($img_tag->[1]{alt} =~ /Poster$/) {
 				$cover = $img_tag->[1]{src};
 				last;
 			}
@@ -523,14 +718,78 @@ sub cover {
 	return $self->{_cover};
 }	
 
+sub top_info {
+	my CLASS_NAME $self = shift;
+	my $forced = shift || 0;
+	if($forced or !$self->{'_top_info'}) {
+		my $parser = $self->_parser(FORCED);
+		while(my $tag = $parser->get_tag('div')) {
+			last if $tag->[1]->{class} && $tag->[1]->{class} eq 'article highlighted';
+		}
+		my $text = $parser->get_trimmed_text('span');
+		my @top_items = split /\s?\|\s?/, $text;
+		$self->{_top_info} = \@top_items;
+	}
+	return $self->{_top_info};
+}
+
+=item recommendation_movies()
+
+Return a list of recommended movies for specified one as a hash where each key is a movie ID in IMDB and
+value - movie's title:
+
+	$recommendation_movies = $film->recommendation_movies();
+
+For example, the list of recommended movies for Troy will be similar to that:
+
+	__DATA__
+	$VAR1 = {                                                                                                                                 
+          '0416449' => '300',                                                                                                             
+          '0167260' => 'The Lord of the Rings: The Return of the King',                                                                   
+          '0442933' => 'Beowulf',                                                                                                         
+          '0320661' => 'Kingdom of Heaven',                                                                                               
+          '0172495' => 'Gladiator'                                                                                                        
+        };   
+
+=cut
+
+sub recommendation_movies {
+	my CLASS_NAME $self = shift;
+	my $forced = shift || 0;
+
+	if($forced) {
+		my $parser = $self->_parser(FORCED);
+
+		while(my $tag = $parser->get_tag('h2')) {
+			my $text = $parser->get_text();
+			last if $text =~ /recommendations/i;
+		}
+		
+		my %result = ();
+		while(my $tag = $parser->get_tag()) {
+			last if $tag->[0] eq '/table';
+			
+			my $text = $parser->get_text();
+			if($tag->[0] eq 'a' && $text && $tag->[1]{href} =~ /tt(\d+)/) {
+				$result{$1} = $text;
+			}
+		}
+		
+		$self->{_recommendation_movies} = \%result;
+	}
+
+	return $self->{_recommendation_movies};
+}
+
 =item directors()
 
 Retrieve film directors list each element of which is hash reference -
 { id => <ID>, name => <Name> }:
 
 	my @directors = @{ $film->directors() };
-	
+
 =cut
+
 sub directors {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -540,15 +799,16 @@ sub directors {
 		my (@directors, $tag);
 	
 		while($tag = $parser->get_tag(MAIN_TAG)) {
-			last if $parser->get_text =~ /^direct(?:ed|or)/i;
+			my $text = $parser->get_text;
+			last if $text =~ /direct(?:ed|or)/i;
 		}
-
+		
 		while ($tag = $parser->get_tag() ) {
 			my $text = $parser->get_text();
 			
 			last if $text =~ /^writ(?:ing|ers)/i or $tag->[0] eq '/div';
 			
-			if($tag->[0] eq 'a' && $tag->[1]{href} && $text !~ /img/i) {
+			if($tag->[0] eq 'a' && $tag->[1]{href} && $text !~ /(img|more)/i) {
 				my($id) = $tag->[1]{href} =~ /(\d+)/;	
 				push @directors, {id => $id, name => $text};
 			}			
@@ -571,6 +831,7 @@ Retrieve film writers list each element of which is hash reference -
 contain a full list!</I>	
 
 =cut
+
 sub writers {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -607,6 +868,7 @@ Retrieve film genres list:
 	my @genres = @{ $film->genres() };
 
 =cut
+
 sub genres {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -621,7 +883,7 @@ sub genres {
 
 		while(my $tag = $parser->get_tag('a')) {
 			my $genre = $parser->get_text;	
-			last unless $tag->[1]{href} =~ /genres/i;
+			last unless $tag->[1]{href} =~ m!/genre/!i;
 			last if $genre =~ /more/i;
 			push @genres, $genre;
 		}	
@@ -635,10 +897,11 @@ sub genres {
 =item tagline()
 
 Retrieve film tagline:
-	
+
 	my $tagline = $film->tagline();
 
 =cut
+
 sub tagline {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -658,45 +921,60 @@ sub tagline {
 
 =item plot()
 
-Retrieve film plot summary:
+Returns a movie plot:
 
-	my $plot = $film->plot();
+	my $plot = $film->plot;
 
 =cut
+
 sub plot {
 	my CLASS_NAME $self = shift;
-	my $forced = shift || 0;
 
-	my ($text);
+	return $self->{_plot};
+}
+
+=item storyline()
+
+Retrieve film plot summary:
+
+	my $storyline = $film->storyline();
+
+=cut
+
+sub storyline {
+	my CLASS_NAME $self = shift;
+	my $forced = shift || 0;
 
 	if($forced) {
 		my $parser = $self->_parser(FORCED);
 
-		while(my $tag = $parser->get_tag(MAIN_TAG)) {
-			last if $parser->get_text =~ /^plot/i;
+		while(my $tag = $parser->get_tag('h2')) {
+			last if $parser->get_text =~ /^storyline$/i;
 		}
-
-		$self->{_plot} = $parser->get_trimmed_text(MAIN_TAG, 'a');
-
-		my $tag = $parser->get_tag('a');
+ 		
+		my $plot = $parser->get_trimmed_text(MAIN_TAG, 'em');
+		$self->{_storyline} = $self->_decode_special_symbols($plot);
 	}	
 
-	return $self->{_plot};
+	return $self->{_storyline};
 }
 
 =item rating()
 
 In scalar context returns film user rating, in array context returns 
-film rating and number of votes:
-	
+film rating, number of votes and info about place in TOP 250 or some other TOP and avards:
+
 	my $rating = $film->rating();
 
 	or
 
-	my($rating, $vnum) = $film->rating();
-	print "RATING: $rating ($vnum votes )";
+	my($rating, $vnum, $avards) = $film->rating();
+	print "RATING: $rating ($vnum votes)";
+
+Note, that $avards is array reference where the first elemen is a TOP info if so, and the next element is other avards - Oscar, nominations and etc	
 
 =cut
+
 sub rating {
 	my CLASS_NAME $self = shift;
 	my ($forced) = shift || 0;
@@ -704,19 +982,22 @@ sub rating {
 	if($forced) {
 		my $parser = $self->_parser(FORCED);
 	
-		while(my $tag = $parser->get_tag(MAIN_TAG)) {
-			last if $parser->get_text =~ /^User Rating/i;
+		while(my $tag = $parser->get_tag('div')) {
+			last if $tag->[1]{class} && $tag->[1]{class} eq 'star-box-details';
 		}
 		
-		my $tag = $parser->get_tag('b');	
-		my $text = $parser->get_trimmed_text('b', '/a');
+		my $text = $parser->get_trimmed_text('/a');
 
-		$self->_show_message("Rating text is [$text]; tag: " . Dumper($tag), 'DEBUG');
-
-		my($rating, $val) = $text =~ m!(\d*\.?\d*)\/.*?(\d*\,?\d*)\s.*?!;
+		my($rating, $val) = $text =~ m!(\d+\.?\d*)/10.*?(\d+,?\d*)!;
 		$val =~ s/\,// if $val;
-
-		$self->{_rating} = [$rating, $val];
+		
+		$self->{_rating} = [$rating, $val, $self->top_info];
+		
+		unless($self->{_plot}) {
+			my $tag = $parser->get_tag('p');
+			my $text = $parser->get_trimmed_text('/p');
+			$self->{_plot} = $text;
+		}
 	}
 
 	return wantarray ? @{ $self->{_rating} } : $self->{_rating}[0];
@@ -734,6 +1015,7 @@ Note: this method retrieves a cast list first billed only!
 </I>
 
 =cut
+
 sub cast {
 	my CLASS_NAME $self = shift;
 	my ($forced) = shift || 0;
@@ -743,24 +1025,25 @@ sub cast {
 		my $parser = $self->_parser(FORCED);
 	
 		while($tag = $parser->get_tag('table')) {
-			last if $tag->[1]->{class} && $tag->[1]->{class} =~ /cast/i;
+			last if $tag->[1]->{class} && $tag->[1]->{class} =~ /^cast_list$/i;
 		}
-		
-		while($tag = $parser->get_tag('a')) {
-			last if $tag->[1]{href} =~ /fullcredits/i;
-			if($tag->[1]{href} && !$tag->[1]{onclick} && $tag->[1]{href} =~ m#name/nm(\d+?)/#) {
-				$person = $parser->get_text;
-				$id = $1;	
-				my $text = $parser->get_trimmed_text('/tr');
-
-				($role) = $text =~ /.*?\s+(.*)$/;			
-				push @cast, {id => $id, name => $person, role => $role};
-			}	
+		while($tag = $parser->get_tag()) {
+			last if $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ /fullcredits/i;
+			if($tag->[0] eq 'td' && $tag->[1]{class} && $tag->[1]{class} eq 'name') {
+				$tag = $parser->get_tag('a');
+				if($tag->[1]{href} && $tag->[1]{href} =~ m#name/nm(\d+?)/#) {
+					$person = $parser->get_text;
+					$id = $1;	
+					my $text = $parser->get_trimmed_text('/tr');
+					($role) = $text =~ /.*?\s+(.*)$/;
+					push @cast, {id => $id, name => $person, role => $role};
+				}
+			}
 		}	
 		
 		$self->{_cast} = \@cast;
 	}
-	
+
 	return $self->{_cast};
 }
 
@@ -775,6 +1058,7 @@ In array context it retrieves all movie's durations:
 	my @durations = $film->duration();
 
 =cut
+
 sub duration {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -786,8 +1070,8 @@ sub duration {
 			my $text = $parser->get_text();
 			last if $text =~ /runtime:/i;
 		}	
-		
-		my @runtime = split /\s+(\/|\|)\s+/, $parser->get_trimmed_text(MAIN_TAG, 'br');
+		my $duration_str = $self->_decode_special_symbols($parser->get_trimmed_text(MAIN_TAG, '/div'));
+		my @runtime = split /\s+(\/|\|)\s+/, $duration_str;
 		
 		$self->{_duration} = \@runtime;		
 	}
@@ -802,6 +1086,7 @@ Retrieve film produced countries list:
 	my $countries = $film->country();
 
 =cut
+
 sub country {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -815,7 +1100,7 @@ sub country {
 		my (@countries);
 		while(my $tag = $parser->get_tag()) {
 
-			if( $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ /countries/i ) {
+			if( $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ m!/country/[a-z]{2}!i ) {
 				my $text = $parser->get_text();
 				$text =~ s/\n//g;
 				push @countries, $text;
@@ -837,6 +1122,7 @@ Retrieve film languages list:
 	my $languages = $film->language();
 
 =cut
+
 sub language {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -847,16 +1133,15 @@ sub language {
 		while ($tag = $parser->get_tag(MAIN_TAG)) {
 			last if $parser->get_text =~ /language/i;
 		}	
-
+		
 		while($tag = $parser->get_tag()) {
-			
-			if( $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ /languages/i ) {
+			if( $tag->[0] eq 'a' && $tag->[1]{href} && $tag->[1]{href} =~ m!/language/[a-z]{2}!i ) {
 				my $text = $parser->get_text();
 				$text =~ s/\n//g;
 				push @languages, $text;
 			} 
 			
-			last if $tag->[0] eq 'br';
+			last if $tag->[0] eq '/div';
 		}
 
 		$self->{_language} = \@languages; 
@@ -875,6 +1160,7 @@ Retrieve AKA information as array, each element of which is string:
 	print map { "$_\n" } @$aka;
 
 =cut
+
 sub also_known_as {
 	my CLASS_NAME $self= shift;
 	unless($self->{_also_known_as}) {
@@ -882,13 +1168,14 @@ sub also_known_as {
 
         while(my $tag = $parser->get_tag(MAIN_TAG)) {
         	my $text = $parser->get_text();
+			$self->_show_message("AKA: $text", 'DEBUG');
             last if $text =~ /^(aka|also known as)/i;
         }
 
-		my $aka = $parser->get_trimmed_text(MAIN_TAG, 'div');
-
-		my @aka = $aka =~ /(.+?\)(?:\s\(.+?\))?)\s?/g;
-
+		my $aka = $parser->get_trimmed_text('span');
+		
+		$self->_show_message("AKA: $aka", 'DEBUG');
+		my @aka = ($aka);
 		$self->{_also_known_as} = \@aka;
 	}	
 	
@@ -902,6 +1189,7 @@ Retrieve a movie trivia:
 	my $trivia = $film->trivia();
 
 =cut
+
 sub trivia {
 	my CLASS_NAME $self = shift;
 
@@ -912,10 +1200,11 @@ sub trivia {
 =item goofs()
 
 Retrieve a movie goofs:
-	
+
 	my $goofs = $film->goofs();
 
 =cut
+
 sub goofs {
 	my CLASS_NAME $self = shift;
 
@@ -930,12 +1219,38 @@ Retrieve a general information about movie awards like 1 win & 1 nomination:
 	my $awards = $film->awards();
 
 =cut	
+
 sub awards {
 	my CLASS_NAME $self = shift;
 
-	$self->{_awards} = $self->_get_simple_prop('awards') unless $self->{_awards};
+	return $self->{_top_info};
+}
 
-	return $self->{_awards};
+=item mpaa_info()
+
+Return a MPAA for the specified move:
+
+	my mpaa = $film->mpaa_info();
+
+=cut
+
+sub mpaa_info {
+	my CLASS_NAME $self = shift;
+	unless($self->{_mpaa_info}) {
+	
+		my $parser = $self->_parser(FORCED);
+
+        while(my $tag = $parser->get_tag(MAIN_TAG)) {
+        	my $text = $parser->get_trimmed_text(MAIN_TAG, '/a');
+            last if $text =~ /^Motion Picture Rating/i;
+        }
+
+		my $mpaa = $parser->get_trimmed_text('/span');
+		$mpaa =~ s/^\)\s//;
+		$self->{_mpaa_info} = $mpaa;
+	}
+
+	return $self->{_mpaa_info};
 }
 
 =item aspect_ratio()
@@ -945,6 +1260,7 @@ Returns an aspect ratio of specified movie:
 	my $aspect_ratio = $film->aspect_ratio();
 
 =cut
+
 sub aspect_ratio {
 	my CLASS_NAME $self = shift;
 
@@ -958,8 +1274,9 @@ sub aspect_ratio {
 Retrieve film user summary:
 
 	my $descr = $film->summary();
-	
+
 =cut
+
 sub summary {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -988,6 +1305,7 @@ Retrieve list of film certifications each element of which is hash reference -
 	my @cert = $film->certifications();
 
 =cut
+
 sub certifications {
 	my CLASS_NAME $self = shift;
 	my $forced = shift || 0;
@@ -1065,18 +1383,50 @@ sub full_plot {
 	return $self->{_full_plot};
 }
 
+sub big_cover {
+	my CLASS_NAME $self = shift;
+
+	unless($self->{'_big_cover_url'}) {
+		unless($self->{'_big_cover_page'}) {
+			my $parser = $self->_parser(FORCED);
+			my $regexp = '^/media/.+/tt' . $self->code . '$';
+			while(my $tag = $parser->get_tag('a')) {
+				$self->_show_message("$regexp --> " . $tag->[1]->{href}, 'DEBUG');
+				if($tag->[1]->{'href'} =~ m!$regexp!) {	
+					$self->{'_big_cover_page'} = $tag->[1]->{'href'};
+					last;
+				}
+			}
+		}
+		if($self->{'_big_cover_page'}) {
+			my $page = $self->_get_page_from_internet('http://' . $self->{'host'} . $self->{'_big_cover_page'});
+			return unless $page;
+
+			my $parser = $self->_parser(FORCED, \$page);
+			while(my $tag = $parser->get_tag('img')) {
+				if($tag->[1]->{'id'} && $tag->[1]->{'id'} eq 'primary-img') {
+					$self->{'_big_cover_url'} = $tag->[1]->{'src'};
+					last;
+				}
+			}
+		}
+	}
+
+	return $self->{_big_cover_url};
+}
+
 =item official_sites()
 
 Returns a list of official sites of specified movie as array reference which contains hashes
 with site information - URL => Site Title:
-	
+
 	my $sites = $film->official_sites();
 	for(@$sites) {
-		my($url, $title) = each %$_;
-		print "Site name - $title; url - $url\n";
+		print "Site name - $_->{title}; url - $_->{url}\n";
 	}
 
 =cut
+
 sub official_sites {
 	my CLASS_NAME $self = shift;
 
@@ -1096,7 +1446,6 @@ sub official_sites {
 
 		my $parser = $self->_parser(FORCED, \$page);
 		while(my $tag = $parser->get_tag()) {
-			#last if $tag->[1]->{id} && $tag->[1]->{id} eq 'tn15title';
 			last if $tag->[0] eq 'ol';
 		}
 
@@ -1116,14 +1465,16 @@ sub official_sites {
 =item release_dates()
 
 Returns a list of release dates of specified movie as array reference:
-	
+
 	my $sites = $film->release_dates();
 	for(@$sites) {
-		my($country, $date) = each %$_;
-		print "Country - $country; release date - $date\n";
+		print "Country - $_->{country}; release date - $_->{date}; info - $_->{info}\n";
 	}
 
+Option info contains additional information about release - DVD premiere, re-release, restored version etc	
+
 =cut
+
 sub release_dates {
 	my CLASS_NAME $self = shift;
 
@@ -1140,31 +1491,78 @@ sub release_dates {
 		}
 
 		my $parser = $self->_parser(FORCED, \$page);
+		# Searching header of release dates table
 		while(my $tag = $parser->get_tag('th')) {
 			last if $tag->[1]->{class} && $tag->[1]->{class} eq 'xxxx';
 		}
-
+		
+		#
+		# The table has three columns. So we parse then one by one and grab their text
+		#
+		my $count = 0;
+		my @dates = ();
 		while(my $tag = $parser->get_tag()) {
-			last if $tag->[0] eq '/table';
-			next unless $tag->[0] eq 'a' and $tag->[1]->{href} =~ /Recent/;
-			my $country = $parser->get_text;
-			my $date_tag = $parser->get_tag('a');
-			my $date = $parser->get_text;
-			my $year_tag = $parser->get_tag('a');
-			my $year = $parser->get_text;
+			last if $tag->[0] eq '/table';			
+			next unless $tag->[0] eq 'td';
 
-			$self->_show_message("country=$country; date=$date, $year", 'DEBUG');
-
-			push @{ $self->{_release_dates} }, {country => $country, date => "$date, $year"};
+			$dates[$count] = $parser->get_trimmed_text('/td');
+			
+			# When rish 3rd column we should store dates into object property
+			if(++$count > 2) {
+				$dates[2] =~ s/\)\s\(/, /g;
+				$dates[2] =~ s/(\(|\))//g;
+				push @{ $self->{_release_dates} }, {country => $dates[0], date => $dates[1], info => $dates[2]};
+				$count = 0;
+			}	
 		}
 	}
 
 	return $self->{_release_dates};
 }
+=item 
+
+Retrieve a list of plot keywords as an array reference:
+
+	my $plot_keywords = $film->plot_keywords();
+	for my $keyword (@$plot_keywords) {
+		print "keyword: $keyword\n";
+	}
+
+=cut
+
+sub plot_keywords {
+	my CLASS_NAME $self = shift;
+	
+	unless($self->{_plot_keywords}) {
+		my $page;
+		$page = $self->_cacheObj->get($self->code . '_keywords') if $self->_cache;
+
+		unless($page) {
+			my $url = "http://". $self->{host} . "/" . $self->{query} .  $self->code . "/keywords";
+			$self->_show_message("URL for sites is $url ...", 'DEBUG');
+
+			$page = $self->_get_page_from_internet($url);
+			$self->_cacheObj->set($self->code.'_keywords', $page, $self->_cache_exp) if $self->_cache;
+		}
+
+		my $parser = $self->_parser(FORCED, \$page);
+		
+		my @keywords = ();
+		while(my $tag = $parser->get_tag('a')) {
+			my $text = $parser->get_text(); 
+			$text = $self->_decode_special_symbols($text);
+			#$self->_show_message("*** $tag->[1]->{href} --> $text ***", 'DEBUG');
+			push @keywords, $text if $tag->[1]->{href} && $tag->[1]->{href} =~ m#/keyword/#;
+		}
+
+		$self->{_plot_keywords} = \@keywords;
+	}
+
+	return $self->{_plot_keywords};
+}
 
 =back
 
-
 =cut
 
 sub DESTROY {
